from pathlib import Path

import pytest

from solar_registry.service.cos_sync import CosSyncService


@pytest.mark.skip(reason="Can only run in local")
def test_sync_meta_data_to_cos() -> None:
    workdir = str(
        (Path(__file__).parent / "testdata" / "stable_index_file_check").resolve()
    )

    cos_sync = CosSyncService(workdir)
    cos_sync.sync_meta_data(force=False)



"""
测试工具模型定义

这里面的模型都是对外体现的
"""

from enum import Enum
from typing import Literal

from pydantic import BaseModel, Field


class ParamChoice(BaseModel):
    value: str
    desc: str


class ParamDef(BaseModel):
    name: str
    value: str
    default: str
    choices: list[ParamChoice] | None = None


class Entry(BaseModel):
    load: str
    run: str


class OsType(str, Enum):
    Linux = "linux"
    Windows = "windows"
    Darwin = "darwin"
    Android = "android"


class ArchType(str, Enum):
    Amd64 = "amd64"
    Arm64 = "arm64"


class LegacySpec(BaseModel):
    res_pkg_url: str = Field(alias="resPkgUrl")


class TestTool(BaseModel):
    __test__ = False

    """
    测试工具模型定义
    """

    schema_version: float = Field(alias="schemaVersion")
    name: str = Field(pattern=r"^[a-zA-Z-]+$")
    description: str = Field(min_length=10, max_length=1000)

    # x.x.x 格式版本
    version: str = Field(pattern=r"^(\d+\.\d+\.\d+|stable)$")
    lang: Literal["python", "golang", "javascript", "java"]
    base_image: str = Field(alias="defaultBaseImage")
    lang_type: Literal["COMPILED", "INTERPRETED"] = Field(alias="langType")
    param_defs: list[ParamDef] | None = Field(None, alias="parameterDefs")
    home_page: str = Field(alias="homePage")
    version_file: str = Field(alias="versionFile")
    index_file: str = Field(alias="indexFile")
    scaffold_repo: str = Field(alias="scaffoldRepo")
    support_os: list[OsType] | None = Field(None, alias="supportOS")
    support_arch: list[ArchType] | None = Field(None, alias="supportArch")
    entry: Entry | None = Field(None, alias="entry")
    git_pkg_url: str = Field("", alias="gitPkgUrl")
    name_zh: str = Field("", alias="nameZh")
    legacy_spec: LegacySpec | None = Field(None, alias="legacySpec")

    def check_valid(self) -> None:
        """
        检查测试工具定义是否合法

        直接在模型中增加非None检查会导致旧版本的测试工具元数据解析报错，所以单独提取一个函数用于校验，需要的时候再调用
        """

        assert self.support_os, "should have supportOS in yaml"
        assert len(self.support_os) > 0, "need at least 1 support OS"

        assert self.support_arch, "should have supportArch in yaml"
        assert len(self.support_arch) > 0, "need at least 1 support arch"

        assert self.git_pkg_url, "should have gitPkgUrl in yaml"

        assert self.name_zh, "should have nameZh in yaml"


class TestToolTarget(BaseModel):
    __test__ = False

    """
    发布包模型定义
    """

    os: OsType
    arch: ArchType
    download_url: str = Field(alias="downloadUrl")
    sha256: str


class StableIndexMetaData(BaseModel):
    """
    稳定版本索引文件
    """

    meta_version: str = Field("1", alias="metaVersion")
    tools: list[TestTool]


class TestToolMetadata(BaseModel):
    __test__ = False

    """
    通过solar-registry生成的最新版本发布元数据

    包含元数据信息和target信息
    """

    meta: TestTool
    target: list[TestToolTarget]


class MetaDataHistory(BaseModel):
    """
    工具元数据版本文件
    """

    meta_version: str = Field("1", alias="metaVersion")
    versions: list[TestToolMetadata]



import os
from pathlib import Path

import yaml
from loguru import logger

from ..model.test_tool import TestTool


def get_testtool(tool_name: str, workdir: str | None) -> TestTool:
    logger.debug(f"querying testtool for {tool_name}")
    workdir = workdir or os.getcwd()
    with open(Path(workdir) / tool_name / "testtool.yaml") as f:
        testtool = TestTool.model_validate(yaml.safe_load(f))
        logger.debug(
            f"loaded testtool: {testtool.model_dump_json(indent=2, exclude_none=True)}"
        )
        testtool.check_valid()
        return testtool


def github_asset_gen(testtool: TestTool) -> str:
    return f"https://github.com/OpenTestSolar/testtool-{testtool.lang}-{testtool.name}/archive/refs/tags/{testtool.version}.tar.gz"



from pathlib import Path

import pytest
from pydantic import ValidationError

from solar_registry.service.testtool import get_testtool
from solar_registry.model.test_tool import OsType, ArchType


def test_validate_correct_pytest_tool() -> None:
    workdir = str((Path(__file__).parent / "testdata").resolve())

    tool = get_testtool("pytest", workdir)

    assert tool.name == "pytest"
    assert tool.version == "0.1.6"
    assert tool.name_zh == "pytest自动化测试"
    assert tool.git_pkg_url == "github.com/OpenTestSolar/testtool-python@main:pytest"

    assert tool.support_os
    assert tool.support_os[0] == OsType.Windows
    assert tool.support_os[1] == OsType.Linux
    assert tool.support_os[2] == OsType.Darwin

    assert tool.support_arch
    assert tool.support_arch[0] == ArchType.Amd64
    assert tool.support_arch[1] == ArchType.Arm64


def test_validate_name_error() -> None:
    workdir = str((Path(__file__).parent / "testdata" / "error_meta_file").resolve())

    with pytest.raises(ValidationError) as ve:
        get_testtool("pytest", workdir)

    assert r"String should match pattern '^[a-zA-Z-]+$'" in str(ve.value)


def test_validate_version_error() -> None:
    workdir = str((Path(__file__).parent / "testdata" / "error_version_file").resolve())

    with pytest.raises(ValidationError) as ve:
        get_testtool("pytest", workdir)

    assert r"String should match pattern '^(\d+\.\d+\.\d+|stable)$'" in str(ve.value)


def test_validate_os_type_error() -> None:
    workdir = str((Path(__file__).parent / "testdata" / "error_os_and_arch").resolve())

    with pytest.raises(ValidationError) as ve:
        get_testtool("pytest", workdir)

    print(ve)

    assert r"Input should be 'linux', 'windows', 'darwin' or 'android'" in str(ve.value)


def test_validate_arch_type_error() -> None:
    workdir = str((Path(__file__).parent / "testdata" / "error_os_and_arch").resolve())

    with pytest.raises(ValidationError) as ve:
        get_testtool("pytest1", workdir)

    print(ve)

    assert r"Input should be 'amd64' or 'arm64'" in str(ve.value)
